@@ -11,18 +11,19 @@
 # limitations under the License.
 
 import datetime
-import docker
 import errno
 import os
 import shutil
 import tarfile
 
+import docker.errors
 import git
 import requests
 from requests import exceptions as requests_exc
 
 from kolla.common import task  # noqa
 from kolla.common import utils  # noqa
+from kolla.engine_adapter import engine
 from kolla.image.utils import Status
 from kolla.image.utils import STATUS_ERRORS
 
@@ -31,21 +32,18 @@ class ArchivingError(Exception):
     pass
 
 
-class DockerTask(task.Task):
-
-    docker_kwargs = docker.utils.kwargs_from_env()
-
-    def __init__(self):
-        super(DockerTask, self).__init__()
-        self._dc = None
+class EngineTask(task.Task):
+    def __init__(self, conf):
+        super(EngineTask, self).__init__()
+        self._ec = None
+        self.conf = conf
 
     @property
-    def dc(self):
-        if self._dc is not None:
-            return self._dc
-        docker_kwargs = self.docker_kwargs.copy()
-        self._dc = docker.APIClient(version='auto', **docker_kwargs)
-        return self._dc
+    def engine_client(self):
+        if self._ec is not None:
+            return self._ec
+        self._ec = engine.getEngineClient(self.conf)
+        return self._ec
 
 
 class PushIntoQueueTask(task.Task):
@@ -70,11 +68,11 @@ class PushError(Exception):
     pass
 
 
-class PushTask(DockerTask):
-    """Task that pushes an image to a docker repository."""
+class PushTask(EngineTask):
+    """Task that pushes an image to a container image repository."""
 
     def __init__(self, conf, image):
-        super(PushTask, self).__init__()
+        super(PushTask, self).__init__(conf)
         self.conf = conf
         self.image = image
         self.logger = image.logger
@@ -89,9 +87,10 @@ class PushTask(DockerTask):
         try:
             self.push_image(image)
         except requests_exc.ConnectionError:
-            self.logger.exception('Make sure Docker is running and that you'
-                                  ' have the correct privileges to run Docker'
-                                  ' (root)')
+            self.logger.exception('Make sure container engine daemon is '
+                                  'running and that you have the correct '
+                                  'privileges to run the '
+                                  'container engine (root)')
             image.status = Status.CONNECTION_ERROR
         except PushError as exception:
             self.logger.error(exception)
@@ -110,7 +109,8 @@ class PushTask(DockerTask):
     def push_image(self, image):
         kwargs = dict(stream=True, decode=True)
 
-        for response in self.dc.push(image.canonical_name, **kwargs):
+        for response in self.engine_client.push(
+                image.canonical_name, **kwargs):
             if 'stream' in response:
                 self.logger.info(response['stream'])
             elif 'errorDetail' in response:
@@ -120,11 +120,11 @@ class PushTask(DockerTask):
         image.status = Status.BUILT
 
 
-class BuildTask(DockerTask):
+class BuildTask(EngineTask):
     """Task that builds out an image."""
 
     def __init__(self, conf, image, push_queue):
-        super(BuildTask, self).__init__()
+        super(BuildTask, self).__init__(conf)
         self.conf = conf
         self.image = image
         self.push_queue = push_queue
@@ -145,8 +145,9 @@ class BuildTask(DockerTask):
         followups = []
         if self.conf.push and self.success:
             followups.extend([
-                # If we are supposed to push the image into a docker
-                # repository, then make sure we do that...
+                # If we are supposed to push the image into a
+                # container image repository,
+                # then make sure we do that...
                 PushIntoQueueTask(
                     PushTask(self.conf, self.image),
                     self.push_queue),
@@ -264,6 +265,14 @@ class BuildTask(DockerTask):
 
     def builder(self, image):
 
+        def _test_malicious_tarball(archive, path):
+            tar_file = tarfile.open(archive, 'r|*')
+            for n in tar_file.getnames():
+                if not os.path.abspath(os.path.join(path, n)).startswith(path):
+                    tar_file.close()
+                    self.logger.error(f'Unsafe filenames in archive {archive}')
+                    raise ArchivingError
+
         def make_an_archive(items, arcname, item_child_path=None):
             if not item_child_path:
                 item_child_path = arcname
@@ -277,8 +286,9 @@ class BuildTask(DockerTask):
                     archives.append(archive_path)
             if archives:
                 for archive in archives:
+                    _test_malicious_tarball(archive, items_path)
                     with tarfile.open(archive, 'r') as archive_tar:
-                        archive_tar.extractall(path=items_path)
+                        archive_tar.extractall(path=items_path)  # nosec
             else:
                 try:
                     os.mkdir(items_path)
@@ -358,15 +368,16 @@ class BuildTask(DockerTask):
 
         buildargs = self.update_buildargs()
         try:
-            for stream in self.dc.build(path=image.path,
-                                        tag=image.canonical_name,
-                                        nocache=not self.conf.cache,
-                                        rm=True,
-                                        decode=True,
-                                        network_mode=self.conf.network_mode,
-                                        pull=pull,
-                                        forcerm=self.forcerm,
-                                        buildargs=buildargs):
+            for stream in \
+                self.engine_client.build(path=image.path,
+                                         tag=image.canonical_name,
+                                         nocache=not self.conf.cache,
+                                         rm=True,
+                                         decode=True,
+                                         network_mode=self.conf.network_mode,
+                                         pull=pull,
+                                         forcerm=self.forcerm,
+                                         buildargs=buildargs):
                 if 'stream' in stream:
                     for line in stream['stream'].split('\n'):
                         if line:
@@ -379,11 +390,19 @@ class BuildTask(DockerTask):
                             self.logger.error('%s', line)
                     return
 
-            if image.status != Status.ERROR and self.conf.squash:
+            if image.status != Status.ERROR and self.conf.squash and \
+               self.conf.engine == engine.Engine.DOCKER.value:
                 self.squash()
-        except docker.errors.DockerException:
+
+        except engine.getEngineException(self.conf) as e:
             image.status = Status.ERROR
-            self.logger.exception('Unknown docker error when building')
+            if isinstance(e, docker.errors.BuildError):
+                for line in e.build_log:
+                    if 'stream' in line:
+                        self.logger.error(line['stream'].strip())
+            self.logger.exception('Unknown container engine '
+                                  'error when building')
+
         except Exception:
             image.status = Status.ERROR
             self.logger.exception('Unknown error when building')
@@ -395,9 +414,9 @@ class BuildTask(DockerTask):
 
     def squash(self):
         image_tag = self.image.canonical_name
-        image_id = self.dc.inspect_image(image_tag)['Id']
+        image_id = self.engine_client.inspect_image(image_tag)['Id']
 
-        parent_history = self.dc.history(self.image.parent_name)
+        parent_history = self.engine_client.history(self.image.parent_name)
         parent_last_layer = parent_history[0]['Id']
         self.logger.info('Parent lastest layer is: %s' % parent_last_layer)
 
