@@ -17,7 +17,6 @@
 UNBUILDABLE_IMAGES = {
     'aarch64': {
         "bifrost-base",        # someone need to get upstream working first
-        "gnocchi-base",        # https://github.com/gnocchixyz/gnocchi/issues/1304 # noqa
         "prometheus-msteams",  # no aarch64 binary
         "prometheus-mtail",    # no aarch64 binary
     },
@@ -25,7 +24,6 @@ UNBUILDABLE_IMAGES = {
     # Issues for SHA1 keys:
     # https://github.com/grafana/grafana/issues/41036
     'centos': {
-        "gnocchi-base",          # https://github.com/gnocchixyz/gnocchi/issues/1304 # noqa
         "hacluster-pcs",         # Missing crmsh package
         "nova-spicehtml5proxy",  # Missing spicehtml5 package
         "ovsdpdk",               # Not supported on CentOS
@@ -33,11 +31,9 @@ UNBUILDABLE_IMAGES = {
     },
 
     'debian': {
-        "gnocchi-base",  # https://github.com/gnocchixyz/gnocchi/issues/1304
     },
 
     'rocky': {
-        "gnocchi-base",          # https://github.com/gnocchixyz/gnocchi/issues/1304 # noqa
         "hacluster-pcs",         # Missing crmsh package
         "nova-spicehtml5proxy",  # Missing spicehtml5 package
         "ovsdpdk",               # Not supported on CentOS
@@ -45,18 +41,13 @@ UNBUILDABLE_IMAGES = {
     },
 
     'ubuntu': {
-        "collectd",      # Missing collectd-core package
-        "gnocchi-base",  # https://github.com/gnocchixyz/gnocchi/issues/1304
-        "telegraf",      # Missing collectd-core package
     },
 
     'ubuntu+aarch64': {
         "barbican-base",  # https://github.com/unbit/uwsgi/issues/2434
-        "gnocchi-base",   # https://github.com/gnocchixyz/gnocchi/issues/1304
     },
 
     'centos+aarch64': {
-        "gnocchi-base",  # https://github.com/gnocchixyz/gnocchi/issues/1304
         "telegraf",      # no binary package
     },
 }
